'use strict';

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _map = require('lodash/map');

var _map2 = _interopRequireDefault(_map);

var _trim = require('lodash/trim');

var _trim2 = _interopRequireDefault(_trim);

var _pick = require('lodash/pick');

var _pick2 = _interopRequireDefault(_pick);

var _filter = require('lodash/filter');

var _filter2 = _interopRequireDefault(_filter);

var _defaultsDeep = require('lodash/defaultsDeep');

var _defaultsDeep2 = _interopRequireDefault(_defaultsDeep);

var _colors = require('colors');

var _colors2 = _interopRequireDefault(_colors);

var _commander = require('commander');

var _commander2 = _interopRequireDefault(_commander);

var _configuration = require('../share/configuration');

var _nginx = require('../builder/nginx');

var _nginx2 = _interopRequireDefault(_nginx);

var _printer = require('../share/printer');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const cwd = _path2.default.basename(require.main.filename);

_commander2.default.command('nginx').description('Generate nginx vhosts config file with modules').option('-c, --config <config>', 'Set module config (Default path/to/nginx.json)').option('-d, --dist <dist>', 'Set destination file').option('-b, --base <base>', 'Set destination base path').option('-p, --port <port>', 'Set webpack develop server port in development').option('--root-path <rootPath>', 'Set variable \'root\' in nginx conf (Default destination folder)').option('--logs-path <logsPath>', 'Set log folder in nginx conf (Default \'base/logs/\')').option('--use-https', 'Use https protocol (Default false)').option('--cert-path', 'Set root cert path (Default base folder)').option('--cert-file', 'Set cert file (Require when --use-https is open)').option('--cert-key', 'Set cert key file (Require when --use-https is true)').action((options = {}) => {
  let startTime = Date.now();
  let nginxModules = _configuration.modules;
  let nginxPort = _configuration.port;

  if (options.hasOwnProperty('config')) {
    let config = _fsExtra2.default.readJSONSync(options.config);
    nginxPort = config.port;
    nginxModules = config.modules;
  }

  options.port = options.hasOwnProperty('port') ? options.port * 1 : nginxPort;

  let modules = (0, _map2.default)(nginxModules, module => {
    let _options = {
      type: module.type || 'module',
      proxy: '127.0.0.1',
      port: nginxPort,
      reserved: (0, _map2.default)(module.reserved, path => (0, _trim2.default)(path, '/'))
    };

    return (0, _defaultsDeep2.default)(_options, module);
  });

  (0, _nginx2.default)(modules, {
    basePath: options.base,
    distFile: options.dist,
    rootPath: options.rootPath,
    logsPath: options.logsPath
  }, (error, stats) => {
    /* istanbul ignore if */
    if (error) {
      throw error;
    }

    let { file, modules } = stats;
    modules = (0, _filter2.default)(modules, ({ domain, proxy }) => domain && proxy);
    modules = (0, _map2.default)(modules, module => (0, _pick2.default)(module, ['domain', 'proxy', 'entry', 'port']));

    (0, _printer.trace)(`${_colors2.default.blue(_colors2.default.bold('Nginx Config Generator'))}`);
    (0, _printer.trace)(`Time: ${_colors2.default.bold(_colors2.default.white(Date.now() - startTime))}ms\n`);

    if ((0, _printer.printStats)(modules)) {
      (0, _printer.trace)(`[${_colors2.default.bold(_colors2.default.green('ok'))}] Nginx config file ${_colors2.default.bold(_colors2.default.green(file))} is generated completed`);
      (0, _printer.trace)(`[${_colors2.default.bold(_colors2.default.blue('info'))}] Remember include it and ${_colors2.default.bold(_colors2.default.green('reolad/restart'))} your nginx server`);
    }
  });
}).on('--help', () => {
  (0, _printer.trace)('  Examples:');
  (0, _printer.trace)(`    $ ${cwd} vhosts`);
  (0, _printer.trace)('');
});