'use strict';

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _indexOf = require('lodash/indexOf');

var _indexOf2 = _interopRequireDefault(_indexOf);

var _colors = require('colors');

var _colors2 = _interopRequireDefault(_colors);

var _commander = require('commander');

var _commander2 = _interopRequireDefault(_commander);

var _configuration = require('../share/configuration');

var _printer = require('../share/printer');

var _compiler = require('../share/compiler');

var _compiler2 = _interopRequireDefault(_compiler);

var _unitest = require('../share/unitest');

var _unitest2 = _interopRequireDefault(_unitest);

var _development = require('../tasks/development');

var _development2 = _interopRequireDefault(_development);

var _production = require('../tasks/production');

var _production2 = _interopRequireDefault(_production);

var _e2e = require('../tasks/e2e');

var _e2e2 = _interopRequireDefault(_e2e);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const cwd = _path2.default.basename(require.main.filename);

const commander = _commander2.default.command('run <mode>').description('Start webpack').option('-c, --config <config>', 'Set webpack config').option('-p, --port <port>', 'Set dev server port').option('-h, --host <host>', 'Set dev server host').option('--https', 'Server run in https protocol').action((mode, options) => {
  let serverPort = options.port || _configuration.port;
  let serverHost = options.host || '127.0.0.1';
  let serverHttps = options.https || false;
  let configFile;

  if (options.hasOwnProperty('config')) {
    configFile = _path2.default.resolve(_configuration.rootDir, options.config);

    if (!_fsExtra2.default.existsSync(configFile)) {
      (0, _printer.trace)(_colors2.default.red(`Webpack config file ${_colors2.default.bold(configFile)} is not exists`));
      process.exit(0);
    }
  }

  if ((0, _indexOf2.default)(['dev', 'develop', 'development'], mode) !== -1) {
    process.env.DEVELOP = 1;

    let options = { port: serverPort, host: serverHost, https: serverHttps };
    configFile ? (0, _compiler2.default)(configFile, options) : (0, _development2.default)(options);
    return;
  }

  if ((0, _indexOf2.default)(['prod', 'product', 'production'], mode) !== -1) {
    process.env.PRODUCTION = 1;
    configFile ? (0, _compiler2.default)(configFile) : (0, _production2.default)();
    return;
  }

  if ((0, _indexOf2.default)(['e2e', 'test'], mode) !== -1) {
    process.env.PRODUCTION = 1;
    configFile ? (0, _unitest2.default)(configFile) : (0, _e2e2.default)();
    return;
  }

  commander.help();
  process.exit(0);
}).on('--help', () => {
  (0, _printer.trace)('  Examples:');
  (0, _printer.trace)(`    $ ${_colors2.default.blue(`${cwd} develop`)}`);
  (0, _printer.trace)(`    $ ${_colors2.default.blue(`${cwd} product`)}`);
  (0, _printer.trace)(`    $ ${_colors2.default.blue(`${cwd} e2e`)}`);
  (0, _printer.trace)('');
});