'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Rules = exports.Plugins = exports.ResolveModules = exports.Entries = undefined;

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

exports.generateEnteries = generateEnteries;
exports.generateFavicons = generateFavicons;
exports.generateSprites = generateSprites;
exports.generateSVGSprites = generateSVGSprites;

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _map = require('lodash/map');

var _map2 = _interopRequireDefault(_map);

var _clone = require('lodash/clone');

var _clone2 = _interopRequireDefault(_clone);

var _assign = require('lodash/assign');

var _assign2 = _interopRequireDefault(_assign);

var _without = require('lodash/without');

var _without2 = _interopRequireDefault(_without);

var _isArray = require('lodash/isArray');

var _isArray2 = _interopRequireDefault(_isArray);

var _indexOf = require('lodash/indexOf');

var _indexOf2 = _interopRequireDefault(_indexOf);

var _forEach = require('lodash/forEach');

var _forEach2 = _interopRequireDefault(_forEach);

var _isObject = require('lodash/isObject');

var _isObject2 = _interopRequireDefault(_isObject);

var _filter = require('lodash/filter');

var _filter2 = _interopRequireDefault(_filter);

var _defaults = require('lodash/defaults');

var _defaults2 = _interopRequireDefault(_defaults);

var _autoprefixer = require('autoprefixer');

var _autoprefixer2 = _interopRequireDefault(_autoprefixer);

var _postcssPxtorem = require('postcss-pxtorem');

var _postcssPxtorem2 = _interopRequireDefault(_postcssPxtorem);

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _cleanWebpackPlugin = require('clean-webpack-plugin');

var _cleanWebpackPlugin2 = _interopRequireDefault(_cleanWebpackPlugin);

var _spritesheetTemplates = require('spritesheet-templates');

var _spritesheetTemplates2 = _interopRequireDefault(_spritesheetTemplates);

var _webpackSpritesmith = require('webpack-spritesmith');

var _webpackSpritesmith2 = _interopRequireDefault(_webpackSpritesmith);

var _webpackSvgstorePlugin = require('webpack-svgstore-plugin');

var _webpackSvgstorePlugin2 = _interopRequireDefault(_webpackSvgstorePlugin);

var _faviconsWebpackPlugin = require('favicons-webpack-plugin');

var _faviconsWebpackPlugin2 = _interopRequireDefault(_faviconsWebpackPlugin);

var _extractTextWebpackPlugin = require('extract-text-webpack-plugin');

var _extractTextWebpackPlugin2 = _interopRequireDefault(_extractTextWebpackPlugin);

var _htmlWebpackPlugin = require('html-webpack-plugin');

var _htmlWebpackPlugin2 = _interopRequireDefault(_htmlWebpackPlugin);

var _copyWebpackPlugin = require('copy-webpack-plugin');

var _copyWebpackPlugin2 = _interopRequireDefault(_copyWebpackPlugin);

var _webpackManifestPlugin = require('webpack-manifest-plugin');

var _webpackManifestPlugin2 = _interopRequireDefault(_webpackManifestPlugin);

var _configuration = require('../share/configuration');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const DefinePlugin = _webpack2.default.DefinePlugin;
const { CommonsChunkPlugin } = _webpack2.default.optimize;
const GlobalVariables = (0, _defaults2.default)({ publicPath: (0, _stringify2.default)(_configuration.publicPath) }, _configuration.variables);

let PostCSSPlugins = [(0, _autoprefixer2.default)({
  browsers: ['last 10 version', 'ie >= 9']
})];

(0, _forEach2.default)(_configuration.plugins, ({ name, options }) => {
  if (name === 'postcss-pxtorem') {
    options = (0, _defaults2.default)({}, options, {
      rootValue: 16,
      unitPrecision: 5,
      propList: ['*'],
      propWhiteList: ['*'],
      selectorBlackList: ['html'],
      replace: true,
      mediaQuery: false,
      minPixelValue: 0
    });

    PostCSSPlugins.push((0, _postcssPxtorem2.default)(options));
  }
});

/**
 * Entries definitions
 */
const Entries = exports.Entries = {};

/**
 * reolve path definitions
 */
const ResolveModules = exports.ResolveModules = [_path2.default.join(_configuration.execDir, 'node_modules'), _path2.default.join(_configuration.rootDir, 'node_modules'), _configuration.srcDir, _configuration.tmpDir];

/**
 * Plugins definitions
 */
const Plugins = exports.Plugins = [
/**
 * Clean generate folders
 * run it first to reset the project.
 */
new _cleanWebpackPlugin2.default([_configuration.tmpDir, _configuration.distDir], {
  root: '/',
  verbose: true,
  dry: false
}),

/**
 * Define some global variables
 */
new DefinePlugin(GlobalVariables),

/**
 * Extract common modules
 * to reduce code duplication
 */
new CommonsChunkPlugin({
  name: 'vendor',
  minChunks: module => /node_modules/.test(module.resource)
}),

/**
 * Extract style file
 * Inline styles can be externally optimized for loading
 */
new _extractTextWebpackPlugin2.default({
  filename: 'styles/[name].[contenthash].css',
  allChunks: true
}),

/**
 * Copy files
 */
new _copyWebpackPlugin2.default([{
  from: _path2.default.join(_configuration.srcDir, 'assets/panels/**'),
  to: _path2.default.join(_configuration.distDir, 'assets/panels/'),
  flatten: true
},
/**
 * 临时解决办法, 更新后删除
 * docs: https://github.com/jantimon/favicons-webpack-plugin/issues/52
 */
{
  from: _path2.default.join(_configuration.srcDir, 'assets/favicons/favicon.ico'),
  to: _path2.default.join(_configuration.distDir, '/favicon.ico'),
  flatten: true
}]),

/**
 * Generate all static resource
 * manifest json file
 */
new _webpackManifestPlugin2.default()];

/**
 * Generate some compile tasks
 */
generateEnteries(Plugins, Entries);
generateFavicons(Plugins);
generateSVGSprites(Plugins, Entries);
let spriteGenerated = generateSprites(Plugins);

/**
 * loader and rules definitions
 */
const Rules = exports.Rules = [{
  test: /\.html$/,
  use: [{
    loader: 'html-loader',
    options: {
      attrs: ['img:src', 'img:ng-src']
    }
  }]
}, {
  test: /\.pug$/,
  use: [{
    loader: 'pug-loader'
  }]
},
/**
 * docs:
 * - https://github.com/webpack-contrib/extract-text-webpack-plugin/issues/263
 */
{
  test: /\.(sass|s?css)$/,
  use: _extractTextWebpackPlugin2.default.extract({
    fallback: 'style-loader',
    use: [{
      loader: 'css-loader',
      options: {
        minimize: true
      }
    }, {
      loader: 'postcss-loader',
      options: {
        plugins: PostCSSPlugins
      }
    }, {
      loader: 'sass-loader',
      options: {
        includePaths: ResolveModules,
        data: [spriteGenerated ? '@import "sprites";' : ''].join('\n')
      }
    }]
  })
}, {
  test: /\.js$/,
  use: [{
    loader: 'ng-annotate-loader',
    options: {
      es6: true
    }
  },
  /**
   * babel@6.0.0 break the .babelrc file
   * so configure presets below
   * docs:
   * - https://github.com/babel/babel-loader/issues/166
   */
  {
    loader: 'babel-loader',
    options: {
      babelrc: _path2.default.join(_configuration.rootDir, './.babelrc')
    }
  }],
  exclude: [/node_modules/]
},
/**
 * 少于 1K 图片用 base64
 * url-loader 依赖 file-loader
 */
{
  test: /\.(jpe?g|png|gif|woff|woff2|eot|ttf|svg)$/i,
  use: [{
    loader: 'url-loader',
    options: {
      limit: 1 * 1024,
      name: 'panels/[name].[hash].[ext]'
    }
  }]
}];

let WebpackDefaultSettings = {
  entry: Entries,
  output: {
    path: _configuration.distDir,
    publicPath: _configuration.publicPath,
    filename: '[name].js'
  },
  node: {
    __filename: true,
    __dirname: true
  },
  module: {
    rules: Rules
  },
  resolve: {
    modules: ResolveModules
  },
  resolveLoader: {
    modules: ResolveModules
  },
  plugins: Plugins
};

let rootConfigFile = _path2.default.join(_configuration.rootDir, 'webpack.ngwp.config.babel.js');
if (_fsExtra2.default.existsSync(rootConfigFile)) {
  let options = _fsExtra2.default.readJSONSync(_path2.default.join(_configuration.execDir, '.babelrc'));
  require('babel-register', options);
  WebpackDefaultSettings = require(rootConfigFile).default((0, _clone2.default)(WebpackDefaultSettings));
}

/**
 * Webpack Setting
 */
exports.default = WebpackDefaultSettings;

/**
 * Auto generate entries
 * Generate entries dependent on folder (src/entry/{folder})
 * And entry js file must be named 'index.js'
 */

function generateEnteries(plugins, entries) {
  if (!(0, _isArray2.default)(plugins)) {
    throw new Error('Parameter plugins must be a array.');
  }

  if (!(0, _isObject2.default)(entries)) {
    throw new Error('Parameter entries must be a object.');
  }

  let modules = [];
  (0, _forEach2.default)(_configuration.modules, module => {
    if (!module.entry) {
      return;
    }

    let file = _path2.default.join(_configuration.rootDir, module.entry);
    if (!_fsExtra2.default.existsSync(file)) {
      throw new Error(`Entry file ${module.entry} is not found`);
    }

    let dir = _path2.default.dirname(file);
    let name = _path2.default.basename(dir);

    modules.push({ file, dir, name });
  });

  if (modules.length === 0) {
    throw new Error('Entry is not found');
  }

  let names = (0, _map2.default)(modules, 'name');
  (0, _forEach2.default)(modules, ({ file, dir, name }) => {
    entries[name] = ['babel-polyfill', file];

    /**
     * reanme entry html
     */
    let options = {
      filename: _path2.default.join(_configuration.distDir, `${name}.html`),
      serviceWorker: '/service-worker.js'

      /**
       * use template when then template file exists
       */
    };let templateFile = _path2.default.join(dir, 'index.pug');
    _fsExtra2.default.existsSync(templateFile) && (0, _assign2.default)(options, { template: templateFile });

    /**
     * clean other static resources
     */
    let excludeChunks = (0, _without2.default)(names, name);
    (0, _assign2.default)(options, { excludeChunks });

    let htmlPlugin = new _htmlWebpackPlugin2.default(options);
    plugins.push(htmlPlugin);

    return true;
  });
}

/**
 * auto split logo task
 * if logo file not exists, this task will not be executed.
 */
function generateFavicons(plugins) {
  if (!(0, _isArray2.default)(plugins)) {
    throw new Error('Parameter plugins must be a array.');
  }

  let logoFile = _path2.default.join(_configuration.srcDir, './assets/panels/logo.png');
  if (_fsExtra2.default.existsSync(logoFile)) {
    let statsFile = 'favicon/iconstats.json';
    let plugin = new _faviconsWebpackPlugin2.default({
      title: _configuration.name,
      logo: logoFile,
      prefix: 'favicon/[hash]/',
      emitStats: true,
      statsFilename: statsFile,
      persistentCache: true,
      inject: true,
      background: '#fff',
      icons: {
        android: true,
        appleIcon: true,
        appleStartup: false,
        coast: false,
        favicons: true,
        firefox: true,
        opengraph: false,
        twitter: false,
        yandex: false,
        windows: false
      }
    });

    plugins.push(plugin);

    return true;
  }

  return false;
}

/**
 * Auto concat sprite image
 * if the sprite folder (src/assets/sprites/images/) not exists,
 * this task will not be excuted.
 */
function generateSprites(plugins) {
  if (!(0, _isArray2.default)(plugins)) {
    throw new Error('Parameter plugins must be a array.');
  }

  let spriteDir = _path2.default.join(_configuration.srcDir, './assets/sprites/images');
  let files = _fsExtra2.default.readdirSync(spriteDir);
  let images = (0, _filter2.default)(files, file => /(\.png|\.gif|\.jpg)$/.test(file));

  if (images.length === 0) {
    return false;
  }

  let spriteTemplate = _path2.default.join(spriteDir, 'sprite.scss.template.handlebars');
  if (_fsExtra2.default.existsSync(spriteDir) && _fsExtra2.default.lstatSync(spriteDir).isDirectory() && _fsExtra2.default.existsSync(spriteTemplate)) {
    let source = _fsExtra2.default.readFileSync(spriteTemplate, 'utf8');
    _spritesheetTemplates2.default.addHandlebarsTemplate('spriteScssTemplate', source);

    let plugin = new _webpackSpritesmith2.default({
      src: {
        cwd: spriteDir,
        glob: '**/*.{png,gif,jpg}'
      },
      target: {
        image: _path2.default.join(_configuration.tmpDir, 'sprites.png'),
        css: [[_path2.default.join(_configuration.tmpDir, 'sprites.scss'), {
          format: 'spriteScssTemplate'
        }]]
      },
      apiOptions: {
        cssImageRef: '~sprites.png'
      },
      spritesmithOptions: {
        functions: true,
        padding: 10
      }
    });

    plugins.push(plugin);

    return true;
  }

  return false;
}

/**
 * Auto concat svg sprite image
 * if the sprite folder (src/assets/sprites/svg/) not exists,
 * this task will not be excuted.
 */
function generateSVGSprites(plugins, entries) {
  if (!(0, _isArray2.default)(plugins)) {
    throw new Error('Parameter plugins must be a array.');
  }

  let spriteDir = _path2.default.join(_configuration.srcDir, './assets/sprites/svg');
  let spriteTemplate = _path2.default.join(spriteDir, 'svgstore.config.js');

  if (_fsExtra2.default.existsSync(spriteDir) && _fsExtra2.default.lstatSync(spriteDir).isDirectory() && _fsExtra2.default.existsSync(spriteTemplate)) {
    let files = _fsExtra2.default.readdirSync(spriteDir);
    let images = (0, _filter2.default)(files, file => /\.svg$/.test(file));

    if (images.length === 0) {
      return false;
    }

    (0, _assign2.default)(Entries, { svgstore: spriteTemplate });

    let plugin = new _webpackSvgstorePlugin2.default({
      prefix: 'sp-svg-',
      svgoOptions: {
        plugins: [{ removeComments: true }, { removeMetadata: true }, { removeTitle: true }, { removeDesc: true }, { removeUselessDefs: true }, { removeXMLNS: true }, { minifyStyles: true }, { cleanupIDs: true }, { removeEmptyText: true }, { convertColors: true }, { convertPathData: true }, { convertTransform: true }, { removeUnknownsAndDefaults: true }, { removeUnusedNS: true },
        /**
         * svg in webkit old browser, it not support use (reference)
         * it must use '<use xlink:href="url#id"></use>'
         * and because svgo(https://github.com/svg/svgo) do not set
         * 'xmlns:xlink="http://www.w3.org/1999/xlink"', so it make
         * origin svg content with use tag lack 'namespace' 'prefix',
         * and it make svg display success.
         *
         * SVG 在 webkit 低版本浏览器中不支持内联 use
         * 必须使用 <use xlink:href="url#id"></use>
         * 又因为 svgo 并没有设置 xmlns:xlink="http://www.w3.org/1999/xlink"
         * 因此会使原本 svg 内含有 use 缺少 namespace prefix 的问题, 导致没法兼容加载,
         * 导致外部不能成功导入
         *
         * Error Code:
         * This page contains the following errors:
         * error on line 1 at column 15734: Namespace prefix xlink for href on use is not defined
         * Below is a rendering of the page up to the first error.
         *
         * Browser: Chrome 48.0.2564.23:
         * Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N)
         * AppleWebKit/537.36 (KHTML, like Gecko)
         * Chrome/48.0.2564.23
         * Mobile Safari/537.36
         * wechatdevtools/0.7.0
         * MicroMessenger/6.3.22
         * webview/0
         *
         * Docs : https://github.com/svg/svgo/blob/master/docs/how-it-works/en.md#3-plugins
         * API  : https://github.com/svg/svgo/blob/master/docs/how-it-works/en.md#32-api
         */
        {
          downwardCompatible: {
            type: 'perItem',
            active: true,
            description: 'Backward compatibility, <use> add attribute xmlns:xlink="http://www.w3.org/1999/xlink"',
            params: {},
            fn(item) {
              if (item.isElem('use') && !hasAttr(item, 'xmlns:xlink')) {
                setAttr(item, 'xmlns:xlink', 'http://www.w3.org/1999/xlink');
              }
            }
          }
        }]
      }
    });

    plugins.push(plugin);
    return true;
  }

  return false;

  function hasAttr(item, name) {
    let [prefix, local] = name.split(':');
    return (0, _indexOf2.default)(item.attrs, { name, prefix, local }) !== -1;
  }

  function setAttr(item, name, value) {
    let [prefix, local] = name.split(':');
    item.attrs[name] = { value, name, prefix, local };
  }
}