'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (file, options = {}, callback) {
  if (!_fsExtra2.default.existsSync(file)) {
    throw new Error(`Config file ${file} is not exists.`);
  }

  let config = require(file);
  config = config.__esModule ? config.default : config;

  let compiler = (0, _webpack2.default)(config);
  options = (0, _defaultsDeep2.default)(options, { watch: false });

  if (options.watch === true) {
    let serverPort = options.port || _configuration.port;
    let serverHost = options.host || '127.0.0.1';
    let serverHttps = (0, _isObject2.default)(options.https) ? options.https || {} : options.https === true ? {} : false;

    let serverConfig = (0, _defaults2.default)({}, config.devServer, {
      // It suppress error shown in console, so it has to be set to false.
      quiet: false,
      // It suppress everything except error, so it has to be set to false as well
      // to see success build.
      noInfo: false,
      stats: {
        // only warning and error informations
        // docs: https://webpack.js.org/configuration/stats/
        colors: true,
        warnings: true,
        errors: true,
        errorDetails: true,

        version: false,
        assets: false,
        cached: false,
        cachedAssets: false,
        modules: false,
        moduleTrace: false,
        chunks: false,
        chunkModules: false,
        chunkOrigins: false,
        children: false,
        hash: false,
        timings: false
      },
      port: serverPort,
      host: serverHost,
      https: serverHttps,
      disableHostCheck: (0, _isBoolean2.default)(options.disableHostCheck) ? options.disableHostCheck : true,
      watchOptions: {
        ignored: /node_modules/,
        aggregateTimeout: 300,
        poll: 1000
      }
    });

    let server = new _webpackDevServer2.default(compiler, serverConfig);
    server.listen(serverPort, serverHost, error => {
      if (error) {
        throw error;
      }

      let server = `http${serverHttps ? 's' : ''}://${serverHost}:${serverPort}`;

      (0, _printer.trace)('');
      (0, _printer.trace)(`Ngwp version at ${_colors2.default.cyan.bold(_package2.default.version)}`);
      (0, _printer.trace)(`Project is running at ${_colors2.default.cyan.bold(server)}`);
      (0, _printer.trace)(`Webpack output is served from ${_colors2.default.cyan.bold((0, _get2.default)(config, 'output.publicPath'))}`);
      (0, _printer.trace)(`Content not from webpack is served from ${_colors2.default.cyan.bold((0, _get2.default)(config, 'output.path'))}`);
      (0, _printer.trace)(''); // 空行
      (0, _printer.printStats)(_configuration.modules);
      (0, _printer.trace)(`✨ Please make sure nginx config file is generated! You can also run script ${_colors2.default.magenta.bold('ngwp nginx')}`);
      (0, _printer.trace)('');
    });

    return;
  }

  compiler.run(function (error, stats) {
    if (error) {
      throw error;
    }

    /* eslint no-console:off */
    // let message = stats.toString({
    //   chunks: false,
    //   colors: true
    // })

    // trace(message)
  });
};

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _get = require('lodash/get');

var _get2 = _interopRequireDefault(_get);

var _isObject = require('lodash/isObject');

var _isObject2 = _interopRequireDefault(_isObject);

var _isBoolean = require('lodash/isBoolean');

var _isBoolean2 = _interopRequireDefault(_isBoolean);

var _defaults = require('lodash/defaults');

var _defaults2 = _interopRequireDefault(_defaults);

var _defaultsDeep = require('lodash/defaultsDeep');

var _defaultsDeep2 = _interopRequireDefault(_defaultsDeep);

var _colors = require('colors');

var _colors2 = _interopRequireDefault(_colors);

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _webpackDevServer = require('webpack-dev-server');

var _webpackDevServer2 = _interopRequireDefault(_webpackDevServer);

var _configuration = require('./configuration');

var _printer = require('./printer');

var _package = require('../../package.json');

var _package2 = _interopRequireDefault(_package);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }